/*
 * author: Darren Bounds <darren@sfhn.net>
 * copyright: Copyright (C) 2002 by Darren Bounds
 * license: This software is under GPL version 2 of license
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * packit official page at http://www.intrusense.com/~dbounds
 */

#ifndef _PACKIT_HEADERS
#define _PACKIT_HEADERS

#define PACKIT_ARP_H             0x1c    /* ARP header:          28 bytes */
#define PACKIT_DNS_H             0xc     /* DNS header base:     12 bytes */
#define PACKIT_ETH_H             0xe     /* Etherner header:     14 bytes */
#define PACKIT_ICMPV4_H          0x4     /* ICMP header base:     4 bytes */
#define PACKIT_ICMPV6_H          0x08    /* ICMP6 header base:    8 bytes */
#define PACKIT_ICMPV4_ECHO_H     0x8     /* ICMP_ECHO header:     8 bytes */
#define PACKIT_ICMPV4_MASK_H     0xc     /* ICMP_MASK header:    12 bytes */
#define PACKIT_ICMPV4_UNREACH_H  0x8     /* ICMP_UNREACH header:  8 bytes */
#define PACKIT_ICMPV4_TIMXCEED_H 0x8     /* ICMP_TIMXCEED header: 8 bytes */
#define PACKIT_ICMPV4_REDIRECT_H 0x8     /* ICMP_REDIRECT header: 8 bytes */
#define PACKIT_ICMPV4_TS_H       0x14    /* ICMP_TIMESTAMP headr:20 bytes */
#define PACKIT_IPV4_H            0x14    /* IP header:           20 bytes */
#define PACKIT_IPV6_H            0x28    /* IPv6 header:         40 bytes */
#define PACKIT_TCP_H             0x14    /* TCP header:          20 bytes */
#define PACKIT_UDP_H             0x8     /* UDP header:           8 bytes */

#endif /* _PACKIT_HEADERS */

unsigned short interval;
unsigned short burst;

unsigned short payload_s;
char *payload;

struct iphdr_opts
{
    unsigned short protocol;		
    unsigned short version;			/* ip header version */
    unsigned short rand_saddr;      		 

    char *saddr;
    char *daddr;

    /* IPv4 header options */
    unsigned long v4saddr_nbo; 		/* ipv4 src address network byte order */ 
    unsigned long v4daddr_nbo; 		/* ipv4 dst address network byte order */

    unsigned short id;       			/* ipv4 id number */
    unsigned short ttl;	 		/* ipv4 time to live */

    long tos;				/* ipv4 type of service */

    /* IPv6 header options */
    struct libnet_in6_addr v6saddr_nbo; /* ipv6 src address network byte order */
    struct libnet_in6_addr v6daddr_nbo; /* ipv6 dst address network byte order */

    unsigned short hl;                 	/* ipv6 hop length */
    unsigned short plen;	 	/* ipv6 payload length */
    unsigned short fl;			/* flow label */
    unsigned short pri;			/* priority */

} iphdr_o;

struct tcphdr_opts
{
    unsigned short th_sport;		/* tcp src port */
    unsigned short th_dport;		/* tcp dst port */

    unsigned short th_urg;		/* tcp urg flag */
    unsigned short th_ack;		/* tcp ack flag */ 
    unsigned short th_psh;		/* tcp psh flag */
    unsigned short th_rst;		/* tcp rst flag */
    unsigned short th_syn;		/* tcp syn flag */
    unsigned short th_fin;		/* tcp fin flag */

    unsigned short th_urp;		/* tcp urg pointer */
    unsigned short th_win; 		/* tcp window size */	
    unsigned long th_ackn;		/* tcp ack number */
    unsigned long th_seqn;		/* tcp seq number */
} thdr_o;

struct udphdr_opts
{
    unsigned short uh_sport;   		/* udp src port */
    unsigned short uh_dport;		/* udp dst port */
    unsigned short uh_ulen;		/* udp length */
    unsigned short uh_sum;     		/* udp checksum */
} uhdr_o;

struct icmphdr_opts
{
    unsigned short ih_type;    		/* icmp4 type */
    unsigned short ih_code;		/* icmp4 code */ 
    unsigned short ih_id;		/* icmp4 id */ 
    unsigned short ih_seq;              /* icmp4 seq */
} ihdr_o;

struct enethdr_opts
{
    char *str_saddr;		/* source ethernet address string */
    char *str_daddr; 	  	/* destination ethernet address string */

    unsigned char eh_saddr[6];		/* source ethernet address */
    unsigned char eh_daddr[6];		/* destination ethernet address */ 
} ehdr_o; 

#ifndef _INJECT_FUNC
#define _INJECT_FUNC

void            injection_stats();
void            injection_clean_exit(int sig);
void		packit_build_ethernet_addr(char *ethstr, int sd);
void 		display_injection(unsigned short init_type, unsigned short count);
unsigned char * packit_retrieve_rand_v4_addr();
libnet_t *	packit_build_ethernet(libnet_t *packet_d);
libnet_t * 	packit_build_ip(libnet_t *packet_d);
libnet_t *	packit_build_tcp(libnet_t *packet_d);
libnet_t *	packit_build_udp(libnet_t *packet_d);
libnet_t *	packit_build_icmp(libnet_t *packet_d);

#endif /* _INJECT_FUNC */
