/*
 * author: Darren Bounds <darren@sfhn.net>
 * copyright: Copyright (C) 2002 by Darren Bounds
 * license: This software is under GPL version 2 of license
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * packit official page at http://www.intrusense.com/~dbounds
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>

#include "../include/packit.h" 
#include "../include/inject.h"
#include "../include/error.h"

libnet_t *
packit_build_ethernet(libnet_t *packet_d)
{
    int hex16;

    struct libnet_ether_addr *enet;

    if(ehdr_o.str_saddr == NULL)
    {
        if((enet = libnet_get_hwaddr(packet_d)) == NULL)
        {
             fprintf(stderr, "\nError: Unable to determine ethernet address: %s\n", libnet_geterror(packet_d));
             exit(EXIT_FAILURE);
        }

        strcpy(ehdr_o.eh_saddr, enet->ether_addr_octet);

            for (hex16 = 0; hex16 < 6; hex16++)
                ehdr_o.eh_saddr[hex16] = enet->ether_addr_octet[hex16];
    }
    else
    {
        packit_build_ethernet_addr(ehdr_o.str_saddr, 1);
    }

    if(ehdr_o.str_daddr == NULL)
        fprintf(stderr, "\nWarning: Using null destination ethernet address. Packets may not reach destination.\n");
    else
        packit_build_ethernet_addr(ehdr_o.str_daddr, 2);

    if(libnet_build_ethernet(
        ehdr_o.eh_daddr,
        ehdr_o.eh_saddr,
        ETHERTYPE_IP,
        NULL,
        0,
        packet_d,
        0) == -1)
    {
        fprintf(stderr, "\nError: Unable to build Ethernet header: %s\n", libnet_geterror(packet_d));
        exit(EXIT_FAILURE);
    }

    return packet_d;
}

void
packit_build_ethernet_addr(char *ethstr, int sd)
{
     int i;
     long hex16;
     char *sep, *eptr, *brkb;

     sep = ":";
     i = 0;

     for (eptr = strtok_r(ethstr, sep, &brkb);
          eptr;
          eptr = strtok_r(NULL, sep, &brkb))
     {
         hex16 = strtol(eptr, 0, 16);

         if(sd == 1)
             ehdr_o.eh_saddr[i] = hex16;
         else if(sd == 2)
             ehdr_o.eh_daddr[i] = hex16;
       
         i++;
     }

     return;
}
