/*
 * author: Darren Bounds <darren@sfhn.net>
 * copyright: Copyright (C) 2002 by Darren Bounds
 * license: This software is under GPL version 2 of license
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * packit official page at http://www.intrusense.com/~dbounds
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>

#include "../include/packit.h" 
#include "../include/inject.h"
#include "../include/error.h"


unsigned char *
packit_retrieve_rand_v4_addr()
{
    int oct, oct_count; 
    unsigned char *ip;
 
    ip = malloc(sizeof(unsigned char) * 16);

    for(oct_count = 1; oct_count < 5; oct_count++)
    {
        srand(time(0) ^ getpid() * oct_count);
        oct = rand() % 255;

        if(oct_count != 1)
            snprintf(ip, 16, "%s.%d", ip, oct);
        else
            snprintf(ip, 16, "%d", oct);
    }

    return ip;
}

libnet_t *
packit_build_ip(libnet_t *packet_d)
{
    static int rand_id;

    if(iphdr_o.daddr == NULL)
    {
         fprintf(stderr, "\nError: No destination IP address defined\n");
         exit(EXIT_FAILURE);
    }

    if(iphdr_o.rand_saddr)
       iphdr_o.saddr = packit_retrieve_rand_v4_addr();

    if(iphdr_o.saddr == NULL)
    {
        if((iphdr_o.v4saddr_nbo = libnet_get_ipaddr4(packet_d)) == -1)
        {
            fprintf(stderr, "\nError: Unable to determine local IP address: %s\n", libnet_geterror(packet_d));
            exit(EXIT_FAILURE);
        }
        else
        {
            iphdr_o.saddr = libnet_addr2name4(iphdr_o.v4saddr_nbo, 0);
        }
    }

    if((iphdr_o.v4saddr_nbo = libnet_name2addr4(packet_d, iphdr_o.saddr, 1)) == -1)
    {
        fprintf(stderr, "\nError: Invalid source IP address: %s\n", iphdr_o.saddr);
        exit(EXIT_FAILURE);
    }

    if((iphdr_o.v4daddr_nbo = libnet_name2addr4(packet_d, iphdr_o.daddr, 1)) == -1)
    {
        fprintf(stderr, "\nError: Invalid source IP address: %s\n", iphdr_o.daddr);
        exit(EXIT_FAILURE);
    }

    if(!iphdr_o.id || rand_id)
    {
        rand_id = 1;
        iphdr_o.id = libnet_get_prand(LIBNET_PRu16);
    }

    if(iphdr_o.version == 4)
        if(iphdr_o.rand_saddr)
            iphdr_o.saddr = packit_retrieve_rand_v4_addr();

    if(libnet_build_ipv4(
        PACKIT_IPV4_H + hdrlen + payload_s,
        iphdr_o.tos,
        iphdr_o.id,
        0,
        iphdr_o.ttl,
        iphdr_o.protocol,
        0,
        iphdr_o.v4saddr_nbo,
        iphdr_o.v4daddr_nbo,
        NULL,
        0,
        packet_d,
        0) == -1)
    {
        fprintf(stderr, "\nError: Unable to build IPv4 header: %s\n", libnet_geterror(packet_d));
        exit(EXIT_FAILURE);
    }
  
    return packet_d;
}
