/*
 * author: Darren Bounds <darren@sfhn.net>
 * copyright: Copyright (C) 2002 by Darren Bounds
 * license: This software is under GPL version 2 of license
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * packit official page at http://www.intrusense.com/~dbounds
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "../include/packit.h" 
#include "../include/inject.h"
#include "../include/error.h"

int 
packit_retrieve_tcp_flags()
{
    int th_flags = 0;

    if(thdr_o.th_urg)
        th_flags |= TH_URG;

    if(thdr_o.th_ack)
        th_flags |= TH_ACK;

    if(thdr_o.th_psh)
        th_flags |= TH_PUSH;

    if(thdr_o.th_rst)
        th_flags |= TH_RST;

    if(thdr_o.th_syn)
        th_flags |= TH_SYN;

    if(thdr_o.th_fin)
        th_flags |= TH_FIN;

    return th_flags;
}

libnet_t *
packit_build_tcp(libnet_t *packet_d)
{
    int th_flags = 0;
    int rand_sport = 0;

    hdrlen = PACKIT_TCP_H;

    if(!thdr_o.th_sport || rand_sport)
    {
        rand_sport = 1;  
        thdr_o.th_sport =  libnet_get_prand(LIBNET_PRu16); 
    }

    th_flags = packit_retrieve_tcp_flags(th_flags);

    if(libnet_build_tcp(
        thdr_o.th_sport, 
        thdr_o.th_dport, 
        thdr_o.th_seqn, 
        thdr_o.th_ackn, 
        th_flags, 
        thdr_o.th_win, 
        0, 
        thdr_o.th_urp, 
        PACKIT_TCP_H + payload_s, 
        payload, 
        payload_s, 
        packet_d, 
        0) == -1)
    {
        fprintf(stderr, "\nError: Unable to build TCP header: %s\n", libnet_geterror(packet_d)); 
        exit(EXIT_FAILURE);
    } 

    return packet_d;
}
