/*
 * author: Darren Bounds <darren@sfhn.net>
 * copyright: Copyright (C) 2002 by Darren Bounds
 * license: This software is under GPL version 2 of license
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * packit official page at http://www.intrusense.com/~dbounds
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "../include/packit.h"
#include "../include/capture.h"
#include "../include/error.h"

int hdrlen;

struct icmp_d
check_icmp_code(struct libnet_icmpv4_hdr *icmphdr, int icmp_type_int)
{
    struct icmp_d icmp;

    if(icmp_type_int == 3)
    {
        switch(icmphdr->icmp_code)
        {
            case 0:
                strcpy(icmp.icmp_code, "net");
                icmp.icmp_code_int = icmphdr->icmp_code;
                break;

            case 1:
                strcpy(icmp.icmp_code, "host");
                icmp.icmp_code_int = icmphdr->icmp_code; 
                break;

            case 2:
                strcpy(icmp.icmp_code, "protocol");
                icmp.icmp_code_int = icmphdr->icmp_code;
                break;

            case 3:
                strcpy(icmp.icmp_code, "port");
                icmp.icmp_code_int = icmphdr->icmp_code;
                break;

            case 4:
                strcpy(icmp.icmp_code, "need frag");
                icmp.icmp_code_int = icmphdr->icmp_code;
                break;

            case 5: 
                strcpy(icmp.icmp_code, "src fail");
                icmp.icmp_code_int = icmphdr->icmp_code;
                break;

            case 6:
                strcpy(icmp.icmp_code, "net unknown");
                icmp.icmp_code_int = icmphdr->icmp_code;
                break;

            case 7: 
                strcpy(icmp.icmp_code, "host unknown");
                icmp.icmp_code_int = icmphdr->icmp_code;
                break;

            case 8: 
                strcpy(icmp.icmp_code, "isolated");
                icmp.icmp_code_int = icmphdr->icmp_code; 
                break;

            case 9: 
                strcpy(icmp.icmp_code, "net prohib");
                icmp.icmp_code_int = icmphdr->icmp_code;
                break;

            case 10:
                strcpy(icmp.icmp_code, "host prohib");
                icmp.icmp_code_int = icmphdr->icmp_code; 
                break;

            case 11: 
                strcpy(icmp.icmp_code, "tos net");
                icmp.icmp_code_int = icmphdr->icmp_code;
                break;

            case 12: 
                strcpy(icmp.icmp_code, "tos host");
                icmp.icmp_code_int = icmphdr->icmp_code;
                break;

            case 13: 
                strcpy(icmp.icmp_code, "filter prohib");
                icmp.icmp_code_int = icmphdr->icmp_code;
                break;

            case 14: 
                strcpy(icmp.icmp_code, "host precedence");
                icmp.icmp_code_int = icmphdr->icmp_code;
                break;

            case 16: 
                strcpy(icmp.icmp_code, "precedence cutoff");
                icmp.icmp_code_int = icmphdr->icmp_code;
                break;
        }
    }
    else if(icmp_type_int == 5)
    {
        switch(icmphdr->icmp_code)
        {
            case 0: 
                strcpy(icmp.icmp_code, "network");
                icmp.icmp_code_int = icmphdr->icmp_code;
                break;

            case 1: 
                strcpy(icmp.icmp_code, "host");
                icmp.icmp_code_int = icmphdr->icmp_code;
                break;

            case 2:
                strcpy(icmp.icmp_code, "tos net");
                icmp.icmp_code_int = icmphdr->icmp_code;
                break;

            case 3:
                strcpy(icmp.icmp_code, "tos host");
                icmp.icmp_code_int = icmphdr->icmp_code;
                break;
        }
    }
    else if(icmp_type_int == 11)
    {
        switch(icmphdr->icmp_code)
        {
            case 0: 
                strcpy(icmp.icmp_code, "transit");
                icmp.icmp_code_int = icmphdr->icmp_code;
                break;

            case 1: 
                strcpy(icmp.icmp_code, "reassembly");
                icmp.icmp_code_int = icmphdr->icmp_code;
                break;
        }
    }
    else if(icmp_type_int == 12)
    {
        switch(icmphdr->icmp_code)
        {
            case 0:
                strcpy(icmp.icmp_code, "bad IP header");
                icmp.icmp_code_int = icmphdr->icmp_code;
                break;

            case 1:
                strcpy(icmp.icmp_code, "missing option");
                icmp.icmp_code_int = icmphdr->icmp_code; 
                break;
        }
    }
    else
    {
        strcpy(icmp.icmp_code, "unknown");
        icmp.icmp_code_int = icmphdr->icmp_code;
    }

    return icmp;
}

struct icmp_d
check_icmp_type(struct libnet_icmpv4_hdr *icmphdr)
{
    struct icmp_d icmp, icmp_code;

    memset(&icmp, 0, sizeof(icmp));
    memset(&icmp_code, 0, sizeof(icmp_code));

    switch(icmphdr->icmp_type)
    {
        case 0:
            strcpy(icmp.icmp_type, "echo reply");
            icmp.icmp_type_int = icmphdr->icmp_type;
            break;

        case 3:
            strcpy(icmp.icmp_type, "unreachable");
            icmp.icmp_type_int = icmphdr->icmp_type;
            icmp_code = check_icmp_code(icmphdr, icmphdr->icmp_type);
            strcpy(icmp.icmp_code, icmp_code.icmp_code);
            icmp.icmp_code_int = icmp_code.icmp_code_int;
            break;

        case 4:
            strcpy(icmp.icmp_type, "source quench");
            icmp.icmp_type_int = icmphdr->icmp_type;
            break;

        case 5:
            strcpy(icmp.icmp_type, "redirect");
            icmp.icmp_type_int = icmphdr->icmp_type;
            break;

        case 8:
            strcpy(icmp.icmp_type, "echo request");
            icmp.icmp_type_int = icmphdr->icmp_type;
            break;

        case 9:
            strcpy(icmp.icmp_type, "router advert");
            icmp.icmp_type_int = icmphdr->icmp_type;
            break;

        case 10:
            strcpy(icmp.icmp_type, "router selection");
            icmp.icmp_type_int = icmphdr->icmp_type; 
            break;

        case 11:
            strcpy(icmp.icmp_type, "time exceed");
            icmp.icmp_type_int = icmphdr->icmp_type;
            icmp_code = check_icmp_code(icmphdr, icmphdr->icmp_type);
            strcpy(icmp.icmp_code, icmp_code.icmp_code);
            icmp.icmp_code_int = icmp_code.icmp_code_int;
            break;

        case 12:
            strcpy(icmp.icmp_type, "param problem");
            icmp.icmp_type_int = icmphdr->icmp_type;
            icmp_code = check_icmp_code(icmphdr, icmphdr->icmp_type);
            strcpy(icmp.icmp_code, icmp_code.icmp_code);
            icmp.icmp_code_int = icmp_code.icmp_code_int;
            break;

        case 13:
            strcpy(icmp.icmp_type, "timestamp");
            icmp.icmp_type_int = icmphdr->icmp_type;
            break;

        case 14:
            strcpy(icmp.icmp_type, "timestamp reply");
            icmp.icmp_type_int = icmphdr->icmp_type;
            break;

        case 15:
            strcpy(icmp.icmp_type, "info request");
            icmp.icmp_type_int = icmphdr->icmp_type;
            break;

        case 16:
            strcpy(icmp.icmp_type, "info reply");
            icmp.icmp_type_int = icmphdr->icmp_type;
            break;

        case 17:
            strcpy(icmp.icmp_type, "address mask request");
            icmp.icmp_type_int = icmphdr->icmp_type;
            break;

        case 18:
            strcpy(icmp.icmp_type, "address mask reply");
            icmp.icmp_type_int = icmphdr->icmp_type;
            break;

        case 30:
            strcpy(icmp.icmp_type, "traceroute");
            icmp.icmp_type_int = icmphdr->icmp_type;
            break;

        default:
            strcpy(icmp.icmp_type, "unknown");
            icmp.icmp_type_int = icmphdr->icmp_type;
            icmp_code = check_icmp_code(icmphdr, icmphdr->icmp_type);
            strcpy(icmp.icmp_code, icmp_code.icmp_code);
            icmp.icmp_code_int = icmp_code.icmp_code_int;
            break;
    }

    return icmp;
}

void
display_icmp_data(unsigned char *packet, char *time_stamp, struct libnet_ethernet_hdr *ehdr, struct libnet_ipv4_hdr *iphdr)
{
    struct icmp_d icmp;
    struct libnet_icmpv4_hdr *icmphdr;
    struct in_addr ip_src, ip_dst;

    bzero(&ip_src, sizeof(ip_src));
    bzero(&ip_dst, sizeof(ip_dst));

    icmphdr = (struct libnet_icmpv4_hdr *)(packet + PACKIT_IPV4_H + hdrlen);
    icmp = check_icmp_type(icmphdr);

    if(verbose < 2) 
    {
        fprintf(stderr, "\n");
        fprintf(stderr, "%s icmp ", time_stamp);

        if(linklayer)
        {
            fprintf(stderr, "%x:%x:%x:%x:%x:%x > ",
                ehdr->ether_shost[0], ehdr->ether_shost[1], ehdr->ether_shost[2], ehdr->ether_shost[3], ehdr->ether_shost[4], ehdr->ether_shost[5]);

            fprintf(stderr, "%x:%x:%x:%x:%x:%x ",
                ehdr->ether_shost[0], ehdr->ether_shost[1], ehdr->ether_shost[2], ehdr->ether_shost[3], ehdr->ether_shost[4], ehdr->ether_shost[5]);
       }

       fprintf(stderr, "%s > %s: type: %s(%d) ",
           libnet_addr2name4(iphdr->ip_src.s_addr, resolve),
           libnet_addr2name4(iphdr->ip_dst.s_addr, resolve),
           icmp.icmp_type,
           icmp.icmp_type_int);

        if(strlen(icmp.icmp_code) > 0)
            fprintf(stderr, "code: %s(%d) ", icmp.icmp_code, icmp.icmp_code_int);

        if(verbose)
            fprintf(stderr, "(ttl %d, id %d)", iphdr->ip_ttl, iphdr->ip_id);
    }
    else
    {
        fprintf(stderr, "Timestamp:   %s GMT\n", time_stamp);

        fprintf(stderr, "ICMP header: Type: %s(%d)  ",
            icmp.icmp_type,
            icmp.icmp_type_int);

        if(strlen(icmp.icmp_code) > 0)
            fprintf(stderr, "Code: %s(%d) ", icmp.icmp_code, icmp.icmp_code_int);

        fprintf(stderr, "\n");

        fprintf(stderr, "IP header:   Src IP: %s  Dst IP: %s",
            libnet_addr2name4(iphdr->ip_src.s_addr, resolve),
            libnet_addr2name4(iphdr->ip_dst.s_addr, resolve));

        fprintf(stderr, "\n");

        fprintf(stderr, "             TTL: %d  ID: %d  TOS: 0x%x  ", iphdr->ip_ttl, iphdr->ip_id, iphdr->ip_tos);

        if(ntohs(iphdr->ip_off) & IP_DF)
            fprintf(stderr, "(DF)");

        fprintf(stderr, "\n");

        fprintf(stderr, "Eth Header:  Src Address: %x:%x:%x:%x:%x:%x  Dst Address: %x:%x:%x:%x:%x:%x",
            ehdr->ether_shost[0], ehdr->ether_shost[1], ehdr->ether_shost[2], ehdr->ether_shost[3], ehdr->ether_shost[4], ehdr->ether_shost[5],
            ehdr->ether_dhost[0], ehdr->ether_dhost[1], ehdr->ether_dhost[2], ehdr->ether_dhost[3], ehdr->ether_dhost[4], ehdr->ether_dhost[5]);

        fprintf(stderr, "\n\n");
    }

    return;
}
