/*
 * author: Darren Bounds <darren@sfhn.net>
 * copyright: Copyright (C) 2002 by Darren Bounds
 * license: This software is under GPL version 2 of license
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * packit official page at http://www.intrusense.com/~dbounds
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "../include/packit.h"
#include "../include/capture.h"
#include "../include/error.h"

int hdrlen;

void 
display_udp_data(unsigned char *packet, char *time_stamp, struct libnet_ethernet_hdr *ehdr, struct libnet_ipv4_hdr *iphdr)
{
    struct libnet_udp_hdr *udphdr;
    struct in_addr ip_src, ip_dst;

    bzero(&ip_src, sizeof(ip_src));
    bzero(&ip_dst, sizeof(ip_dst));

    udphdr = (struct libnet_udp_hdr *)(packet + PACKIT_IPV4_H + hdrlen);

    if(verbose < 2)
    {
        fprintf(stderr, "\n");
        fprintf(stderr, "%s udp ", time_stamp);

        if(linklayer)
        {
            fprintf(stderr, "%x:%x:%x:%x:%x:%x > ",
                ehdr->ether_shost[0], ehdr->ether_shost[1], ehdr->ether_shost[2], ehdr->ether_shost[3], ehdr->ether_shost[4], ehdr->ether_shost[5]);

            fprintf(stderr, "%x:%x:%x:%x:%x:%x ",
                ehdr->ether_shost[0], ehdr->ether_shost[1], ehdr->ether_shost[2], ehdr->ether_shost[3], ehdr->ether_shost[4], ehdr->ether_shost[5]);
        }

        fprintf(stderr, "%s.%u > %s.%u: ulen %d ", 
            libnet_addr2name4(iphdr->ip_src.s_addr, resolve), 
            htons(udphdr->uh_sport), 
            libnet_addr2name4(iphdr->ip_dst.s_addr, resolve), 
            htons(udphdr->uh_dport), 
            udphdr->uh_ulen);

        if(verbose)
            fprintf(stderr, "(ttl %d, id %d, tos 0x%x)", iphdr->ip_ttl, iphdr->ip_id, iphdr->ip_tos);
    }
    else
    {
        fprintf(stderr, "Timestamp:   %s GMT\n", time_stamp);

        fprintf(stderr, "UDP header:  Src Port: %u  Dst Port: %u  Len: %d  ",
            htons(udphdr->uh_sport),
            htons(udphdr->uh_dport),
            udphdr->uh_ulen); 

        fprintf(stderr, "\n");

        fprintf(stderr, "IP header:   Src IP: %s  Dst IP: %s\n",
            libnet_addr2name4(iphdr->ip_src.s_addr, resolve),
            libnet_addr2name4(iphdr->ip_dst.s_addr, resolve));

        fprintf(stderr, "             TTL: %d  ID: %d  TOS: 0x%x  ", iphdr->ip_ttl, iphdr->ip_id, iphdr->ip_tos);

        if(ntohs(iphdr->ip_off) & IP_DF)
            fprintf(stderr, "(DF)");

        fprintf(stderr, "\n");

        fprintf(stderr, "Eth Header:  Src Address: %x:%x:%x:%x:%x:%x  Dst Address: %x:%x:%x:%x:%x:%x\n",
            ehdr->ether_shost[0], ehdr->ether_shost[1], ehdr->ether_shost[2], ehdr->ether_shost[3], ehdr->ether_shost[4], ehdr->ether_shost[5],
            ehdr->ether_dhost[0], ehdr->ether_dhost[1], ehdr->ether_dhost[2], ehdr->ether_dhost[3], ehdr->ether_dhost[4], ehdr->ether_dhost[5]);

        fprintf(stderr, "\n\n");
    }

    return;
}
