/*
 * author: Darren Bounds <darren@sfhn.net>
 * copyright: Copyright (C) 2002 by Darren Bounds
 * license: This software is under GPL version 2 of license
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * packit official page at http://www.intrusense.com/~dbounds
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>

#include "../include/packit.h" 
#include "../include/inject.h"
#include "../include/error.h"


libnet_t *packet_d;

void
injection_stats()
{
    struct libnet_stats ln_stats;

    libnet_stats(packet_d, &ln_stats);

    fprintf(stderr, "------ Packet Injection Statistics ------\n");
    fprintf(stderr,
        "Packet(s) sent: %ld  "
        "Packet errors: %ld  "
        "Bytes written: %ld  ",
        ln_stats.packets_sent, ln_stats.packet_errors, ln_stats.bytes_written);

    fprintf(stderr, "\n");

    return;
}

void
enter_packet_foundry(unsigned short init_type, unsigned short count)
{
    int packet_c;
    
    signal(SIGTERM, injection_clean_exit);
    signal(SIGINT, injection_clean_exit);
    signal(SIGQUIT, injection_clean_exit);
    signal(SIGHUP, injection_clean_exit);

    fprintf(stderr, "Mode: Packet Injection\n\n");

    if(burst > 100)
    {
        fprintf(stderr, "\nError: Maximum burst rate is 100\n");
        exit(EXIT_FAILURE);
    }

    if(payload != NULL)
        payload_s = strlen(payload); 

    if((packet_d = libnet_init(init_type, device, errbuf)) == NULL)
    {
        fprintf(stderr, "\nError: Unable to initialize packet injection: %s\n", errbuf);
        exit(EXIT_FAILURE);
    }

    for(packet_c = 1; packet_c < count + 1; packet_c++)
    {
        if(iphdr_o.protocol == IPPROTO_TCP)
            packet_d = packit_build_tcp(packet_d);
        else if(iphdr_o.protocol == IPPROTO_UDP)
            packet_d = packit_build_udp(packet_d);
        else if(iphdr_o.protocol == IPPROTO_ICMP)
            packet_d = packit_build_icmp(packet_d);

        packet_d = packit_build_ip(packet_d);

        if(init_type == 0)
            packet_d = packit_build_ethernet(packet_d);

        if(packet_c == 1)
            display_injection(init_type, count);

        if(libnet_write(packet_d) == -1)
        {
            fprintf(stderr, "\nError: Unable to write packet to network: %s\n", libnet_geterror(packet_d));
            exit(EXIT_FAILURE); 
        }

        fprintf(stderr, ".");

        /* we shouldn't have to use this */
        libnet_clear_packet(packet_d);

        if(count > 1 && burst && (packet_c % burst) == 0)
            sleep(1);
        else if(count > 1 && count != packet_c && !burst)
            sleep(interval);
    }
  
    injection_clean_exit(1);

    return;
}

void
injection_clean_exit(int sig)
{
    fprintf(stderr, "\n\n");

    injection_stats();
    libnet_destroy(packet_d);

    fprintf(stderr, "\n");

    exit(EXIT_FAILURE);

    return;
}
