/*
 * author: Darren Bounds <darren@sfhn.net>
 * copyright: Copyright (c) 2002 by Darren Bounds
 * license: This software is under GPL version 2 of license
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * packit official page at http://www.intrusense.com/~dbounds
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "../include/packit.h"
#include "../include/inject.h"
#include "../include/capture.h"
#include "../include/error.h"
#include "../include/version.h"

int opt;
char *optarg;

void
packit_parse_capture_options(int argc, char *argv[])
{
    short count = -1;
    resolve = 1;

    while((opt = getopt(argc, argv, "c:eHi:nr:vw:")) != -1)
    {
        switch(opt)
        {
            case 'c':
                count = (unsigned short)atoi(optarg);
                break;

            case 'e':
                linklayer = 1;
                break;

            case 'H':
                verbose = 2;
                break;

            case 'i':
                device = optarg;
                break;

            case 'w':
                strncpy(write_filename, optarg, MAX_CMDOPT_LEN);
                break;

            case 'r':
                strncpy(read_filename, optarg, MAX_CMDOPT_LEN);
                break;

            case 'v':
                verbose = 1;
                break;

            case 'n':
                resolve = 0;
                break;
        }
    }

    start_packet_capture(argv[optind], count);

    return;
}

void
packit_parse_inject_options(int argc, char *argv[], int ip_version)
{
    unsigned short init_type = 0x01;
    unsigned short count = 1;;

    memset(&iphdr_o, 0, sizeof(iphdr_o));
    memset(&ehdr_o, 0, sizeof(iphdr_o));

    interval = 1;
    payload = NULL;
    payload_s = 0; 

    iphdr_o.ttl = 128;
    iphdr_o.tos = 0x00;
    iphdr_o.version = ip_version;

    ehdr_o.str_saddr = NULL;
    init_type = 0x01;
    opterr = 0;

    while((opt = getopt(argc, argv, "t:")) != -1)
    {
        switch(opt)
        {
            case 't':
                if(!strcasecmp(optarg, "tcp"))
                {
                    goto parse_tcp;
                }
                else if(!strcasecmp(optarg, "udp"))
                {
                    goto parse_udp;
                }
                else if(!strcasecmp(optarg, "icmp"))
                {
                    goto parse_icmp;
                }
                else
                {
                    fprintf(stderr, "Error: Invalid injection protocol\n");
                    usage();
                }
              
                break;

            default:
                optind--;
                goto parse_tcp;

                break;
        }
    }

parse_tcp:
    memset(&thdr_o, 0, sizeof(thdr_o));

    thdr_o.th_win = 1500;
    iphdr_o.protocol = IPPROTO_TCP;
                
    while((opt = getopt(argc, argv, "a:b:c:d:D:e:E:F:h:i:n:p:q:rs:S:T:O:u:w:W:")) != -1)
    {
        switch(opt)
        {
            case 'a':
                thdr_o.th_ackn = (unsigned long)atoi(optarg);
                break;

            case 'b':
                burst = (unsigned short)atoi(optarg); 
                break;

            case 'c':
                count = (unsigned short)atoi(optarg);
                break;

            case 'd':
                iphdr_o.daddr = optarg;
                break;

            case 'D':
                thdr_o.th_dport = (unsigned short)atoi(optarg);
                break;

            case 'e':
                init_type = 0x00;
                ehdr_o.str_saddr = optarg;
                break;

            case 'E':
                init_type = 0x00;
                ehdr_o.str_daddr = optarg;
                break;

            case 'F':
                if(strrchr(optarg, 'U'))
                    thdr_o.th_urg = 1;

                if(strrchr(optarg, 'A'))
                    thdr_o.th_ack = 1;

                if(strrchr(optarg, 'P'))
                    thdr_o.th_psh = 1;

                if(strrchr(optarg, 'R'))
                    thdr_o.th_rst = 1;

                if(strrchr(optarg, 'S'))
                    thdr_o.th_syn = 1;

                if(strrchr(optarg, 'F'))
                    thdr_o.th_fin = 1;

                break;

            case 'h':
                iphdr_o.hl = (unsigned short)atoi(optarg);
                break;

            case 'i':
                device = optarg;
                break;

            case 'n':
                iphdr_o.id = (unsigned short)atoi(optarg);
                break;

            case 'O':
                iphdr_o.tos = (long)atoi(optarg);
                break;

            case 'p':
                payload = optarg; 
                break;

            case 'q':
                thdr_o.th_seqn = (unsigned long)atoi(optarg);
                break;

            case 'r':
                iphdr_o.rand_saddr = 1;
                break;

            case 's':
                iphdr_o.saddr = optarg;
                break;

            case 'S':
                thdr_o.th_sport = (unsigned short)atoi(optarg);
                break;

            case 'T':
                iphdr_o.ttl = (unsigned short)atoi(optarg);
                break;

            case 'u':
                thdr_o.th_urp = (unsigned short)atoi(optarg);
                break;

            case 'w':
                interval = (unsigned short)atoi(optarg);
                break;

            case 'W':
                thdr_o.th_win = (unsigned short)atoi(optarg);
                break;
        }
    }
 
    goto enter_foundry;

parse_icmp:
    memset(&ihdr_o, 0, sizeof(ihdr_o));

    ihdr_o.ih_type = 8; 
    ihdr_o.ih_code = 0;
    ihdr_o.ih_id = libnet_get_prand(LIBNET_PRu16); 
    ihdr_o.ih_seq = libnet_get_prand(LIBNET_PRu16);
    iphdr_o.protocol = IPPROTO_ICMP;

    while((opt = getopt(argc, argv, "b:c:C:d:e:E:i:n:N:O:p:s:Q:rT:w:K:")) != -1)
    {

        switch(opt)
        {
            case 'b':
                burst = (unsigned short)atoi(optarg);
                break;

            case 'c':
                count = (unsigned short)atoi(optarg);
                break;

            case 'i':
                device = optarg;
                break;

            case 'w':
                interval = (unsigned short)atoi(optarg);
                break;

            case 'd':
                iphdr_o.daddr = optarg;
                break;

            case 'e':
                init_type = 0x00;
                ehdr_o.str_saddr = optarg;
                break;

            case 'E':
                init_type = 0x00;
                ehdr_o.str_daddr = optarg;
                break;

            case 'n':
                iphdr_o.id = (unsigned short)atoi(optarg);
                break;

            case 'N':
                ihdr_o.ih_id = (unsigned short)atoi(optarg);
                break;

            case 'O':
                iphdr_o.tos = (long)atoi(optarg);
                break;

            case 'p':
                payload = optarg;
                break;

            case 'Q':
                ihdr_o.ih_seq = (unsigned short)atoi(optarg);
                break;

            case 'r':
                iphdr_o.rand_saddr = 1;
                break;

            case 's':
                iphdr_o.saddr = optarg;
                break;

            case 'T':
                iphdr_o.ttl = (unsigned short)atoi(optarg);
                break;

            case 'C':
                ihdr_o.ih_code = (unsigned short)atoi(optarg); 
                break;

            case 'K':
                ihdr_o.ih_type = (unsigned short)atoi(optarg); 
                break;
        }
    }

    goto enter_foundry;

parse_udp:
    memset(&uhdr_o, 0, sizeof(uhdr_o));
    iphdr_o.protocol = IPPROTO_UDP;

    while((opt = getopt(argc, argv, "b:c:d:D:e:E:i:n:O:p:rs:S:T:w:")) != -1)
    {
        switch(opt)
        {
            case 'b':
                burst = (unsigned short)atoi(optarg);
                break;

            case 'c':
                count = (unsigned short)atoi(optarg);
                break;

            case 'd':
                iphdr_o.daddr = optarg;
                break;

            case 'D':
                uhdr_o.uh_dport = (unsigned short)atoi(optarg);
                break;

            case 'e':
                init_type = 0x00;
                ehdr_o.str_saddr = optarg;
                break;

            case 'E':
                init_type = 0x00;
                ehdr_o.str_daddr = optarg;
                break;

            case 'i':
                device = optarg;
                break;

            case 'n':
                iphdr_o.id = (unsigned short)atoi(optarg);
                break;

            case 'O':
                iphdr_o.tos = (long)atoi(optarg);
                break;

            case 'p':
                payload = optarg;
                break;

            case 'r':
                iphdr_o.rand_saddr = 1;
                break;

            case 's':
                iphdr_o.saddr = optarg;
                break;

            case 'S':
                uhdr_o.uh_sport = (unsigned short)atoi(optarg);
                break;

            case 'T':
                iphdr_o.ttl = (unsigned short)atoi(optarg);
                break;

            case 'w':
                interval = (unsigned short)atoi(optarg);
                break;
        }
    }

    goto enter_foundry;
 
enter_foundry:
    enter_packet_foundry(init_type, count);

    return;
}

int
main(int argc, char *argv[])
{
    if(argc < 2)
        usage();
    else if(getuid() != 0)
    {
        fprintf(stderr, "\nError: You're not root\n");
        exit(EXIT_FAILURE);
    }
    else
    {
        opterr = 0;

        while((opt = getopt(argc, argv, "m:")) != -1)
        {
            opterr = 1;

            switch(opt)
            {
                case 'm':
                    if(!strncasecmp(optarg, "capture", 7) || !strncasecmp(optarg, "c", 1))
                    {
                        packit_parse_capture_options(argc, argv);
                    }
                    else if(!strncasecmp(optarg, "inject", 6) || !strncasecmp(optarg, "i", 1))
                    {
                        packit_parse_inject_options(argc, argv, 4);
                    }
                    else
                    {
                        fprintf(stderr, "Error: Invalid runtime mode\n");
                        usage();
                    }

                    break;

                default: 
                    optind--;
                    packit_parse_inject_options(argc, argv, 4);
                    break;
            }
        }
    }

    exit(EXIT_SUCCESS);
}        
